@@ -1,30 +1,20 @@
-    def listAllSnapshots(self, flags=0):
+    def listAllSnapshots(self, flags: int = 0) -> List['virDomainSnapshot']:
         """List all snapshots and returns a list of snapshot objects"""
         ret = libvirtmod.virDomainListAllSnapshots(self._o, flags)
         if ret is None:
-            raise libvirtError("virDomainListAllSnapshots() failed", conn=self)
+            raise libvirtError("virDomainListAllSnapshots() failed")
 
-        retlist = list()
-        for snapptr in ret:
-            retlist.append(virDomainSnapshot(self, _obj=snapptr))
+        return [virDomainSnapshot(self, _obj=snapptr) for snapptr in ret]
 
-        return retlist
-
-
-    def listAllCheckpoints(self, flags=0):
+    def listAllCheckpoints(self, flags: int = 0) -> List['virDomainCheckpoint']:
         """List all checkpoints and returns a list of checkpoint objects"""
         ret = libvirtmod.virDomainListAllCheckpoints(self._o, flags)
         if ret is None:
-            raise libvirtError("virDomainListAllCheckpoints() failed", conn=self)
+            raise libvirtError("virDomainListAllCheckpoints() failed")
 
-        retlist = list()
-        for chkptr in ret:
-            retlist.append(virDomainCheckpoint(self, _obj=chkptr))
+        return [virDomainCheckpoint(self, _obj=chkptr) for chkptr in ret]
 
-        return retlist
-
-
-    def createWithFiles(self, files, flags=0):
+    def createWithFiles(self, files: List[int], flags: int = 0) -> 'virDomain':
         """Launch a defined domain. If the call succeeds the domain moves from the
         defined to the running domains pools.
 
@@ -58,30 +48,63 @@
         If the VIR_DOMAIN_START_FORCE_BOOT flag is set, then any managed save
         file for this domain is discarded, and the domain boots from scratch. """
         ret = libvirtmod.virDomainCreateWithFiles(self._o, files, flags)
-        if ret == -1: raise libvirtError ('virDomainCreateWithFiles() failed', dom=self)
+        if ret == -1:
+            raise libvirtError('virDomainCreateWithFiles() failed')
         return ret
 
-    def fsFreeze(self, mountpoints=None, flags=0):
+    def fsFreeze(self, mountpoints: List[str] = None, flags: int = 0) -> int:
         """Freeze specified filesystems within the guest """
         ret = libvirtmod.virDomainFSFreeze(self._o, mountpoints, flags)
-        if ret == -1: raise libvirtError ('virDomainFSFreeze() failed', dom=self)
+        if ret == -1:
+            raise libvirtError('virDomainFSFreeze() failed')
         return ret
 
-    def fsThaw(self, mountpoints=None, flags=0):
+    def fsThaw(self, mountpoints: List[str] = None, flags: int = 0) -> int:
         """Thaw specified filesystems within the guest """
         ret = libvirtmod.virDomainFSThaw(self._o, mountpoints, flags)
-        if ret == -1: raise libvirtError ('virDomainFSThaw() failed', dom=self)
+        if ret == -1:
+            raise libvirtError('virDomainFSThaw() failed')
         return ret
 
-    def getTime(self, flags=0):
+    def getTime(self, flags: int = 0) -> int:
         """Extract information about guest time """
         ret = libvirtmod.virDomainGetTime(self._o, flags)
-        if ret == None: raise libvirtError ('virDomainGetTime() failed', dom=self)
+        if ret is None:
+            raise libvirtError('virDomainGetTime() failed')
         return ret
 
-    def setTime(self, time=None, flags=0):
+    def setTime(self, time: int = None, flags: int = 0) -> int:
         """Set guest time to the given value. @time is a dict containing
         'seconds' field for seconds and 'nseconds' field for nanoseconds """
         ret = libvirtmod.virDomainSetTime(self._o, time, flags)
-        if ret == -1: raise libvirtError ('virDomainSetTime() failed', dom=self)
+        if ret == -1:
+            raise libvirtError('virDomainSetTime() failed')
         return ret
+
+    def FDAssociate(self, name: str, files: List[int], flags: int = 0) -> int:
+        """Associate the array of FDs passed as @fds with the domain object
+        under @name. The FDs are associated as long as the connection used to
+        associated exists and are disposed of afterwards. FD may still be kept
+        open by the hypervisor for as long as it's needed.
+
+        Security labelling (e.g. via the selinux) may be applied on the passed
+        FDs when requiredg for usage by the VM. By default libvirt does not
+        restore the seclabels on the FDs afterwards to avoid keeping it open
+        unnecessarily.
+
+        Restoring of the security label can be requested by passing either
+        VIR_DOMAIN_FD_ASSOCIATE_SECLABEL_RESTORE for a best-effort attempt to
+        restore the security label after use.  Requesting the restore of
+        security label will require that the file descriptors are kept open for
+        the whole time they are used by the hypervisor, or other additional
+        overhead.
+
+        In certain cases usage of the fd group would imply read-only access.
+        Passing VIR_DOMAIN_FD_ASSOCIATE_SECLABEL_WRITABLE in @flags ensures
+        that a writable security label is picked in case when the file
+        represented by the fds may be used in write mode. """
+        ret = libvirtmod.virDomainFDAssociate(self._o, name, files, flags)
+        if ret == -1:
+            raise libvirtError('virDomainFDAssociate() failed')
+        return ret
+
